(* $Id: commandline.ml,v 1.1 2002/11/10 22:56:47 cpaulin Exp $ *)

open Format
open Filename

(*S Messages divers. *)

(* Affiche un message d'aide sur la sortie standard et exit 0. *)
let help () =
  printf
"Usage: prescript  <options and files>\n
Les options sont :
  -about              a propos de prescript
  -help               affiche cette liste d'options
  -no-parse           analyse lexicale seulement
  -no-sem             analyse lexicale et syntaxique seulement
  <file>              compile <file>\n";
  flush stdout;
  exit 0

(* Affiche quelques informations sur le compilateur sur la sortie standard
   et exit 0. *)
let about () =
  printf
"prescript : un compilateur de prescript vers postscript.
version 0.1.
F. De Jaeger, A. Miquel, C. Paulin, C. Renard et J. Signoles\n";
  flush stdout;
  exit 0

(*S Command line. *)

let no_parse_ref = ref false

let no_sem_ref = ref false

(* Parse la ligne de commandes et
   retourne la liste des fichiers a compiler. *)
let files =
  let files_ref = ref [] in
  let rec parse = function
    | [] -> List.rev !files_ref
    | "-about" :: _ -> about ()
    | "-help" :: _ -> help ()
    | "-no-parse" :: tl ->
	no_parse_ref := true;
	parse tl
    | "-no-sem" :: tl ->
	no_sem_ref := true;
	parse tl
    | f :: tl -> 
	files_ref := f :: !files_ref; 
	parse tl
  in parse (List.tl (Array.to_list Sys.argv))

(* Valeurs issues de la ligne de commandes. *)

let no_parse = !no_parse_ref

let no_sem = !no_sem_ref
