(* *)

(*i*)
open Commandline ;;
(*i*)

(*s Le nom du fichier source est donn par la ligne de commande, et
   le nom du fichier objet correspondant s'en dduit en remplaant
   le suffixe \texttt{.pre} par le suffixe \texttt{.ps}. *)

let my_name = Filename.basename Sys.argv.(0) ;;

let try_open_in file =
  try open_in file with _ ->
    Format.eprintf "%s: impossible d'ouvrir `%s'@." my_name file ;
    exit 1 ;;

let try_open_out file =
  try open_out file with _ ->
    Format.eprintf "%s: impossible d'ouvrir `%s'@." my_name file ;
    exit 1 ;;

let print_parse_error src_file lexbuf =
  Format.eprintf "Fichier `%s', caractres %d-%d:@."
    src_file (Lexing.lexeme_start lexbuf) (Lexing.lexeme_end lexbuf) ;
  Format.eprintf "Erreur de syntaxe@." ;
  exit 1

let print_unknown_error err =
  Format.eprintf "Erreur inconnue (%s)@." (Printexc.to_string err) ;
  exit 1 ;;

let compile src_file =
  let obj_file = Filename.chop_suffix src_file ".pre" ^ ".ps" in
  let ch = try_open_in src_file in
  let lexbuf = Lexing.from_channel ch in
	try 
	      if no_parse then
		while (Lexer.token lexbuf <> Parser.EOF) do () done
	      else if no_sem then 
		let _ = Parser.program Lexer.token lexbuf in ()
	      else failwith "Seule l'analyse syntaxique est disponible"
	with 
	  | Parsing.Parse_error -> print_parse_error src_file lexbuf
	  | e -> print_unknown_error e

let _ = try List.iter compile files with _ -> print_endline "toto"; exit 1








