/* */

/* Dans ce module sont dfinies les fonctions d'analyse syntaxique du
   langage.  Ce module, crit avec la syntaxe Yacc (l'entte et les
   actions des rgles de production tant crites en Caml), est destin
    tre compil avec l'utilitaire \texttt{ocamlyacc}. */

%{ 

%}

/*s \textbf{Dclaration des terminaux}

  \medskip
  Dans cette section sont dfinies les associativits des symboles
  terminaux de la syntaxe, par ordre de prcdence croissante.  Afin
  de prvenir certains conflits, on introduit deux symboles fictifs
  [_SEPAR_] (point-virgule de sparation) et [_UMINUS_] (moins unaire).
  \par\bigskip */

/* $\bullet$\quad Symboles divers (dlimiteurs, symboles de ponctuation,
   oprateurs, etc.):\par\vskip-3pt */
%token LPAR RPAR LBRK RBRK LBRC RBRC DOT COMA COLON SEMI COLONEQ
%token PLUS MINUS TIMES SLASH EQ NE LT LE GT GE AND OR EOF

/* $\bullet$\quad Mots-cls du langage:\par\vskip-3pt */
%token ARRAY BEGIN DIV DO DONE END ELSE FALSE FOR FUN IF IN MOD NOT
%token STEP THEN TO TRUE TYPE VAR WHILE

/* $\bullet$\quad Constantes et identificateurs:\par\vskip-3pt */
%token <int> INT
%token <float> FLOAT
%token <char> CHAR
%token <string> STRING
%token <string> IDENT

/* $\bullet$\quad Le symbole d'axiome (non-terminal) et son
   type:\par\vskip-3pt */
%start program
%type <unit> program

/*s\textbf{Dclaration des prcdences}\par\medskip */

/* $\bullet$\quad Constructions du langage:\par\vskip-3pt */
%nonassoc IN
%left SEMI    /* point-virgule de squence */
%left _SEPAR_ /* point-virgule de sparation (terminal fictif)) */
%nonassoc THEN
%nonassoc ELSE
%nonassoc COLONEQ

/* $\bullet$\quad Oprateurs logiques:\par\vskip-3pt */
%left OR
%left AND
%nonassoc NOT

/* $\bullet$\quad Comparaisons:\par\vskip-3pt */
%nonassoc EQ NE LT LE GT GE

/* $\bullet$\quad Oprateurs arithmtiques:\par\vskip-3pt */
%left PLUS MINUS
%left TIMES SLASH DIV MOD
%left _UMINUS_ /* moins unaire (terminal fictif) */

/* $\bullet$\quad Accs  un lment ou  un champ:\par\vskip-3pt */
%left DOT
%%
/*s\textbf{Rgles de production:}

   \medskip\noindent
   $\bullet$\quad Programmes et dclarations: */
program:
| decls EOF {  } ;

decls:
| /* vide */ { }
| decls decl {  } ;

decl:
| TYPE IDENT EQ typexpr { }
| FUN IDENT LPAR opt_param_list RPAR COLON typexpr EQ expr
    { }
| VAR IDENT EQ expr { } ;

/* \par\noindent
   $\bullet$\quad Expressions de type: */
typexpr:
| IDENT { }
| typexpr ARRAY {  }
| LBRC field_decls RBRC {  } ;

field_decls:
| IDENT COLON typexpr { }
| field_decls SEMI IDENT COLON typexpr { } ;

opt_param_list:
| /* vide */ { }
| param_list { } ;

param_list:
| IDENT COLON typexpr { }
| param_list COMA IDENT COLON typexpr { } ;

/* \par\noindent
   $\bullet$\quad Expressions: */
expr:
| LPAR RPAR { }
| TRUE { }
| FALSE { }
| INT { }
| FLOAT { }
| CHAR { }
| STRING { }
| IDENT { }
| IDENT LPAR opt_arg_list RPAR { }
| LPAR expr RPAR { }
| BEGIN expr END { }
| LBRK opt_expr_list RBRK { }
| LBRC field_defs RBRC { }
| MINUS expr %prec _UMINUS_ { }
| NOT expr { }
| expr PLUS expr { }
| expr MINUS expr { }
| expr TIMES expr { }
| expr SLASH expr { }
| expr MOD expr { }
| expr DIV expr { }
| expr EQ expr { }
| expr NE expr { }
| expr LT expr { }
| expr LE expr { }
| expr GT expr { }
| expr GE expr { }
| expr AND expr { }
| expr OR expr { }
| IF expr THEN expr { }
| IF expr THEN expr ELSE expr { }
| WHILE expr DO expr DONE { }
| WHILE expr DO expr error { }
| FOR IDENT EQ expr TO expr DO expr DONE { }
| FOR IDENT EQ expr TO expr STEP expr DO expr DONE { }
| expr DOT LBRK expr RBRK %prec DOT { }
| expr DOT LPAR expr RPAR %prec DOT { }
| expr DOT IDENT %prec DOT { }
| expr COLONEQ expr { }
| expr SEMI expr { }
| VAR IDENT EQ expr IN expr { }
;

opt_arg_list:
| /* vide */ { }
| arg_list { }
;

arg_list:
| expr { }
| arg_list COMA expr { }
;

field_defs:
| IDENT EQ expr %prec _SEPAR_ { }
| field_defs SEMI IDENT EQ expr %prec _SEPAR_ { }
;

opt_expr_list:
| /* vide */ { }
| expr_list { }
;

expr_list:
| expr { } %prec _SEPAR_
| expr_list SEMI expr %prec _SEPAR_ { }
;
